@@ -1,4 +1,10 @@
+"""
+This module is part of the 'web-compressor' package,
+which is released under GPL-3.0-only license.
+"""
+
 from pathlib import Path
+import re
 from typing import Dict, List, Optional, Tuple
 import uuid
 
@@ -20,8 +26,8 @@ class WebCompressor:
         """
         Constructor
 
-        :param base_dir: Path Root directory
-        :param kwargs: Dict[str, Any] Config options
+        :param base_dir: pathlib.Path Root directory
+        :param kwargs: dict Config options
         :return: None
         """
 
@@ -35,64 +41,63 @@ class WebCompressor:
         # Gotta collect 'em all
         self.html_files = self.collect_html()
 
+    def validate(self, file: Path) -> bool:
+        """
+        Checks whether file may be collected
+
+        Skips directories & hidden files
+
+        :param file: pathlib.Path Path to file
+        :return: bool
+        """
+
+        # Exclude ..
+        # (1) .. directories
+        # (2) .. hidden files
+        return not (file.is_dir() or file.stem.startswith("."))
+
+        # NOTE:
+        # Although it might be preferable to exlude all files
+        # below hidden directories, any one-fits-all approach
+        # will (at some point or to some extent) fail
+
     def collect_html(self) -> List[Path]:
         """
         Collects HTM(L) files
 
-        :return: List[Path] Collected HTM(L) files
+        :return: list Collected HTM(L) files
         """
 
-        # Create data arrays for ..
-        html_files: List[Path] = []
+        def is_valid(file: Path) -> bool:
+            """
+            Checks whether file is valid & identifies as HTM(L)
 
-        for file in self.base_dir.glob("**/*"):
-            # Skip ..
-            # (1) .. directories
-            # (2) .. hidden files
-            if file.is_dir() or file.stem.startswith("."):
-                continue
+            :param file: pathlib.Path Path to file
+            :return: bool
+            """
 
-            # NOTE:
-            # Although it might be preferable to exlude all files
-            # below hidden directories, any one-fits-all approach
-            # will (at some point or to some extent) fail
+            return self.validate(file) and get_mime(file.name) == "text/html"
 
-            # If detected as such ..
-            if get_mime(file.name) == "text/html":
-                # .. add HTM(L) file
-                html_files.append(file)
-
-        return html_files
+        return [file for file in self.base_dir.glob("**/*") if is_valid(file)]
 
     def collect_assets(self) -> List[Tuple[Path, str]]:
         """
         Collects web assets & their mediatypes
 
-        :return: List[Tuple[Path, str]] Collected web assets & their mediatypes
+        :return: list Collected web assets & their mediatypes
         """
 
         # Create data array
         assets: List[Tuple[Path, str]] = []
 
         for file in self.base_dir.glob("**/*"):
-            # Skip ..
-            # (1) .. directories
-            # (2) .. hidden files
-            if file.is_dir() or file.stem.startswith("."):
+            # Skip inapt files
+            if not self.validate(file):
                 continue
 
-            # NOTE:
-            # Although it might be preferable to exlude all files
-            # below hidden directories, any one-fits-all approach
-            # will (at some point or to some extent) fail
-
-            # Determine mediatype from file extension
-            if mime_type := get_mime(file.name):
-                # Skip HTM(L) files
-                if mime_type == "text/html":
-                    continue
-
-                # Store file & MIME type
+            # If mediatype indicates something other than HTM(L) file ..
+            if (mime_type := get_mime(file.name)) != "text/html":
+                # .. store file & MIME type
                 assets.append((file, mime_type))
 
         return assets
@@ -101,14 +106,14 @@ class WebCompressor:
         """
         Replaces asset references inside HTML files
 
-        :param assets: List[Tuple[str, str]] References of source & target files
+        :param assets: list References of source & target files
         :return: None
         """
 
         for html_file in self.html_files:
             # Load original HTML contents
-            with html_file.open("r", encoding="utf-8") as f:
-                html = f.read()
+            with html_file.open("r", encoding="utf-8") as file:
+                html = file.read()
 
             for reference in references:
                 # Unpack reference pair
@@ -118,8 +123,8 @@ class WebCompressor:
                 html = html.replace(old, new)
 
             # Write updated contents
-            with html_file.open("w", encoding="utf-8") as f:
-                f.write(html)
+            with html_file.open("w", encoding="utf-8") as file:
+                file.write(html)
 
             LOGGER.info(f"Updated references in {html_file.relative_to(self.base_dir)}")
 
@@ -131,7 +136,7 @@ class WebCompressor:
             - mediatypes
             - options
 
-        :param kwargs: Dict[str, Any] Keyword arguments
+        :param kwargs: dict Keyword arguments
         :return: None
         """
 
@@ -187,7 +192,7 @@ class WebCompressor:
         Valid arguments are:
             - options
 
-        :param kwargs: Dict[str, Any] Keyword arguments
+        :param kwargs: dict Keyword arguments
         :return: None
         """
 
@@ -206,12 +211,12 @@ class WebCompressor:
 
         for html_file in self.html_files:
             # Load original HTML contents
-            with html_file.open("r", encoding="utf-8") as f:
-                html = f.read()
+            with html_file.open("r", encoding="utf-8") as file:
+                html = file.read()
 
             # Write minified contents
-            with html_file.open("w", encoding="utf-8") as f:
-                f.write(obj.minify_string(html, "text/html"))
+            with html_file.open("w", encoding="utf-8") as file:
+                file.write(obj.minify_string(html, "text/html"))
 
             LOGGER.info(f"Minified {html_file.relative_to(self.base_dir)}")
 
@@ -224,12 +229,12 @@ class WebCompressor:
 
         for html_file in self.html_files:
             # Load original HTML contents
-            with html_file.open("r", encoding="utf-8") as f:
-                html = f.read()
+            with html_file.open("r", encoding="utf-8") as file:
+                html = file.read()
 
             # Write minified contents
-            with html_file.open("w", encoding="utf-8") as f:
-                f.write(str(html.prettify()))
+            with html_file.open("w", encoding="utf-8") as file:
+                file.write(str(html.prettify()))
 
             LOGGER.info(f"Prettified {html_file.relative_to(self.base_dir)}")
 
@@ -241,7 +246,7 @@ class WebCompressor:
             - mediatypes
             - quality
 
-        :param kwargs: Dict[str, Any] Keyword arguments
+        :param kwargs: dict Keyword arguments
         :return: None
         """
 
@@ -290,16 +295,31 @@ class WebCompressor:
             # .. update HTML files
             self.update_html(references)
 
-    def convert_images(self, **kwargs) -> None:
+    def restore_file(self, file: Path) -> Optional[Path]:
         """
-        Converts images to AVIF & WebP
+        Restores original filename after minifying & hashing
+
+        :param file: pathlib.Path Path to file
+        :return: pathlib.Path | None Restored file
+        """
+
+        # If filename matched completely ..
+        if restored := re.fullmatch(r"(.*)\.min\.\w*\.[a-z]{3,4}", file.name):
+            # .. provide its base string
+            return Path(file.parents[0], f"{restored.group(1)}{file.suffix}")
+
+        return None
+
+    def convert_images(self, **kwargs) -> Tuple[Path]:
+        """
+        Converts images to AVIF/WebP & updates their references
 
         Valid arguments are:
             - method
             - quality
 
-        :param kwargs: Dict[str, Any] Keyword arguments
-        :return: None
+        :param kwargs: dict Keyword arguments
+        :return: Tuple[Path]
         """
 
         # Attempt to ..
@@ -315,6 +335,9 @@ class WebCompressor:
             # .. abort process
             return
 
+        # Create data array
+        references: List[Tuple[str, str]] = []
+
         # Iterate over web assets
         for asset in self.collect_assets():
             # Unpack asset file & mediatype
@@ -324,16 +347,33 @@ class WebCompressor:
             if not obj.validate(mime_type):
                 continue
 
-            obj.convert_image(
-                file,
-                mime_type,
-                method=kwargs.get("method", 6),
-                quality=kwargs.get("quality", 90),
-            )
+            # Convert image to AVIF/WebP
+            obj.convert_image(file, **kwargs)
+
+            # If original filename can be detected ..
+            if restored := self.restore_file(file):
+                # .. store references
+                references.extend(
+                    [
+                        #  (1) AVIF
+                        self.get_reference(
+                            restored.with_suffix(".avif"), file.with_suffix(".avif")
+                        ),
+                        # (2) WebP
+                        self.get_reference(
+                            restored.with_suffix(".webp"), file.with_suffix(".webp")
+                        ),
+                    ]
+                )
 
             # .. report back
             LOGGER.info(f"Created WebP & AVIF from {file.relative_to(self.base_dir)}")
 
+        # If files changed ..
+        if references:
+            # .. update HTML files
+            self.update_html(references)
+
     def hash_assets(self, **kwargs) -> None:
         """
         Hashes web assets & updates their references
@@ -343,7 +383,7 @@ class WebCompressor:
             - hash_length
             - use_mtime
 
-        :param kwargs: Dict[str, Any] Keyword arguments
+        :param kwargs: dict Keyword arguments
         :return: None
         """
 
@@ -371,15 +411,12 @@ class WebCompressor:
                 file, kwargs.get("hash_length", 10), kwargs.get("use_mtime", False)
             )
 
-            # Get asset references (relative to root)
-            old, new = self.get_reference(file, target)
+            # Store asset references (relative to root)
+            references.append(self.get_reference(file, target))
 
             # Report back
             LOGGER.info(f"Hashed {file.relative_to(self.base_dir)}")
 
-            # Store references
-            references.append((old, new))
-
         # If files changed ..
         if references:
             # .. update HTML files
@@ -404,8 +441,8 @@ class WebCompressor:
 
         for html_file in self.html_files:
             # Load original HTML contents
-            with html_file.open("r", encoding="utf-8") as f:
-                soup = bs4.BeautifulSoup(f, "html.parser")
+            with html_file.open("r", encoding="utf-8") as file:
+                soup = bs4.BeautifulSoup(file, "html.parser")
 
             # Iterate over eligible tag types
             for dtype, attr in dtypes.items():
@@ -413,10 +450,14 @@ class WebCompressor:
                 for tag in soup.find_all(dtype):
                     # Skip tags ..
                     # (1) .. with 'integrity' attribute
-                    # (2) .. without 'src' attribute
+                    # (2) .. not sourcing external file
                     if tag.has_attr("integrity") or not tag.has_attr(attr):
                         continue
 
+                    # Skip 'link' tags not containing stylesheets
+                    if dtype == "link" and get_mime(tag[attr]) != "text/css":
+                        continue
+
                     # Iterate over web assets
                     for asset in assets:
                         # Unpack asset file & mediatype
@@ -439,8 +480,8 @@ class WebCompressor:
                         break
 
             # Write updated contents
-            with html_file.open("w", encoding="utf-8") as f:
-                f.write(str(soup))
+            with html_file.open("w", encoding="utf-8") as file:
+                file.write(str(soup))
 
     def _generate_csp(self, hashes: Dict[str, List[str]], sets: Dict[str, str]) -> str:
         """
@@ -450,9 +491,9 @@ class WebCompressor:
             - inline 'script' & 'style' tags
             - external 'link' & 'script' tags
 
-        :param hashes: Dict[str, List[str]] Hashed nonces
-        :param directives: Dict[str, str] CSP directives
-        :return: str
+        :param hashes: dict Hashed nonces
+        :param sets: dict CSP directives
+        :return: str CSP directives as string for use in 'meta' tag
         """
 
         csp: List[str] = []
@@ -546,7 +587,7 @@ class WebCompressor:
         """
         Validates CSP nonce (or creates one)
 
-        :param nonce: Optional[str] Nonce to be validated
+        :param nonce: str | None Nonce to be validated
         :return: str Valid CSP nonce
         """
 
@@ -573,8 +614,7 @@ class WebCompressor:
 
         For more information, see '_generate_csp()'
 
-        :param sets: Optional[Dict[str, str]] Set of CSP directives
-        :param kwargs: Dict[str, Any] Keyword arguments
+        :param kwargs: dict Keyword arguments
         :return: None
         """
 
@@ -583,8 +623,8 @@ class WebCompressor:
 
         for html_file in self.html_files:
             # Load original HTML contents
-            with html_file.open("r", encoding="utf-8") as f:
-                soup = bs4.BeautifulSoup(f, "html.parser")
+            with html_file.open("r", encoding="utf-8") as file:
+                soup = bs4.BeautifulSoup(file, "html.parser")
 
             # Normalize nonce (if any)
             nonce = self._get_nonce(kwargs.get("nonce"))
@@ -654,18 +694,12 @@ class WebCompressor:
                 ),
             )
 
-            # Determine earliest tag possible, which would be ..
-            if head := soup.head:
-                # .. first child of 'head' tag (if present)
-                first = head.findChildren()[0]
-
-            # .. otherwise ..
-            else:
-                # .. first child of 'body' tag
-                first = soup.html.findChildren()[0]
+            # Add 'meta' tag containing CSP directives
+            # (1) Determine earliest tag possible (either 'head' or 'body')
+            insert_tag = soup.head if soup.select("head") else soup.html
 
             # Insert 'meta' tag before
-            first.insert_before(meta_tag)
+            insert_tag.findChildren()[0].insert_before(meta_tag)
 
             # NOTE:
             # When minifying HTML contents, the 'head' tag is optional
@@ -675,24 +709,29 @@ class WebCompressor:
             # See https://github.com/tdewolff/minify/issues/90
 
             # Write updated contents
-            with html_file.open("w", encoding="utf-8") as f:
-                f.write(str(soup))
+            with html_file.open("w", encoding="utf-8") as file:
+                file.write(str(soup))
 
             LOGGER.info(f"Added CSP to {html_file.relative_to(self.base_dir)}")
 
-    def get_reference(self, source: Path, target: Path) -> Tuple[str, str]:
+    def get_reference(self, source: Path, target: Path) -> Optional[Tuple[str, str]]:
         """
         Turns two path objects into references relative to root directory
 
-        :param source: Path Source file
-        :param target: Path Target file
-        :return: Tuple[str, str] Asset reference
+        :param source: pathlib.Path Source file
+        :param target: pathlib.Path Target file
+        :return: tuple | None Asset reference
         """
 
-        return (
-            str(source.relative_to(self.base_dir)),
-            str(target.relative_to(self.base_dir)),
-        )
+        # Get their relative paths
+        old = str(source.relative_to(self.base_dir))
+        new = str(target.relative_to(self.base_dir))
+
+        # Provide changed files only
+        if old != new:
+            return old, new
+
+        return None
 
 
 __all__ = [
