@@ -1,3 +1,8 @@
+"""
+This module is part of the 'web-compressor' package,
+which is released under GPL-3.0-only license.
+"""
+
 import abc
 import re
 from typing import List, Union
@@ -14,9 +19,9 @@ class Handler(metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def is_ready(self) -> bool:
         """
-        Checks if everything checks out
+        Checks if dependencies are up & running
 
-        :return: bool Setup status
+        :return: bool
         """
 
     def validate(self, mime_type: str) -> bool:
